# -*- coding: utf-8 -*-
import datetime
from django import forms
from django.conf import settings
from django.conf.urls import patterns, url
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy, ugettext as _
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.contrib.admin.actions import delete_selected as delete_selected_
from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.db import router
from django.contrib.admin.util import get_deleted_objects, model_ngettext
from django.utils.encoding import force_text
import autocomplete_light
from jmb.jadmin import AjaxInline, register_inline
from jmb.jadmin import JumboModelAdmin, ConstrainedModelForm
from jmb.core.forms.models import AdminModelForm

from jmb.organization.models import Organization
from jmb.quotation.models import (
    Quotation, CustomerQuotation, SupplierQuotation,
    QuotationRow, CustomerQuotationRow, SupplierQuotationRow,
    VatRow, CustomerVatRow, SupplierVatRow
)
from jmb.product.models import Product
from jmb.payment.models import PaymentForm


class QuotationRowAjaxInline(AjaxInline):
    model = QuotationRow
    template = 'ajax_inline.html'
    list_display = [
        #'get_detail'
        'get_number', 'get_product_detail',
        'm_unit', 'quantity', 'price', 'discount', 'fee',
        'get_vat', 'get_total'
    ]


class QuotationRowForm(ConstrainedModelForm):
    def __init__(self, *args, **kwargs):
        super(QuotationRowForm, self).__init__(*args, **kwargs)
        quotation_id = False
        obj = kwargs.pop('instance', False)
        if obj:
            quotation = obj.quotation
            quotation_id = quotation.id

        if self.data.get("quotation", False):
            quotation_id = self.data.get("quotation", False)

        if self.initial.get("quotation", False):
            quotation_id = self.initial.get("quotation", False)

        if quotation_id:
           quotation = Quotation.objects.get(id=quotation_id)
           self.fields['quotation'] = forms.ModelChoiceField(
               queryset=Quotation.objects.all(), required=False,
               widget=forms.widgets.HiddenInput()
            )

    class Meta:
        model = QuotationRow

    product = forms.ModelChoiceField(
        Product.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('ProductAutocomplete', )
    )
    # TODO: FIXME - occorre filtrare queryset degli oggetti in ADD e EDIT


class QuotationRowAdmin(JumboModelAdmin):
    model = QuotationRow
    form = QuotationRowForm
    fieldsets = (
        (None, {'fields': (
            'status', 'ordering', 'project', 'product',
            ('vat', 'store_cause'), ('quantity', 'price', 'm_unit'),
            ('fee', 'discount'), 'description', 'long_description',
            'quotation',
        ), }),
    )
    readonly_fields = (
        'creator', 'date_create',
        'last_modifier', 'date_last_modify'
    )
    search_fields = ('quotation__number', 'product__code', 'product__name')
    list_display = [
        #'get_detail'
        'get_date_q', 'get_number', 'get_product_detail',
        'quantity', 'price', 'fee', 'discount', 'str_total_income',
        'str_total_fee', 'str_total_discount', 'str_total',
        'get_cause_code', 'get_vat_code'
    ]
    list_display_csv = [
        'get_date_q', 'get_number', 'get_owner_detail', 'get_product_detail', 'quantity',
        'price', 'fee', 'discount', 'str_total_income', 'str_total_fee',
        'str_total_discount', 'str_total', 'get_cause_code', 'get_vat_code'
    ]
    can_delete = True


    def delete_view(self, request, object_id, extra_context=None):
        quotation_row = QuotationRow.objects.get(id=object_id)
        quotation = quotation_row.quotation

        if not quotation.can_be_delete():
            messages.error(request, _("Quotation already ordered. It's not deletable"))
            return HttpResponseRedirect('..')
        return super(QuotationRowAdmin, self).delete_view(request, object_id, extra_context)

    def get_total(self, obj):
        return (obj.price - obj.discount) * obj.quantity + obj.fee

    get_total.short_description = _('Total')

    def get_vat(self, obj):
        return obj.vat.aliquot_perc * (obj.price - obj.discount) / 100

    get_vat.short_description = _('Vat')

    def get_date_q(self, obj):
        return obj.quotation.date_q.strftime("%d/%m/%Y")

    get_date_q.short_description = _('Date')
    get_date_q.admin_order_field = 'quotation__date_q'

    def get_number(self, obj):
        return obj.quotation.number or ""

    get_number.short_description = _('Number')
    get_number.admin_order_field = 'quotation__number'

    def get_owner_detail(self, obj):
        owner = obj.quotation.owner
        recipient = obj.quotation.recipient or ""
        contact = obj.quotation.contact or ""
        if recipient == owner:
            recipient = ""
        return "%s<br/>%s<br/>%s" % (owner, recipient, contact)

    get_owner_detail.short_description = _("Bill to")
    get_owner_detail.allow_tags = True
    get_owner_detail.admin_order_field = 'quotation__owner'

    def get_product_detail(self, obj):
        product = obj.description or obj.product
        return """<b><a href="%s/">%s</a></b>""" % (obj.id, product)

    get_product_detail.short_description = _("Product")
    get_product_detail.allow_tags = True
    get_product_detail.admin_order_field = 'description'

    def get_cause_code(sel, obj):
        return obj.store_cause.code

    get_cause_code.short_description = _('SC')

    def get_vat_code(self, obj):
        return obj.vat.code

    get_vat_code.short_description = _('VC')

    def str_total_income(self, obj):
        return str("%.2f" % obj.get_total_income())

    str_total_income.short_description = _('Total Income')

    def str_total_fee(self, obj):
        return str("%.2f" % obj.get_total_fee())

    str_total_fee.short_description = _('Total Fee')

    def str_total_discount(self, obj):
        return str("%.2f" % obj.get_total_discount())

    str_total_discount.short_description = _('Total Discount')

    def str_total(self, obj):
        return str("%.2f" % obj.get_total())

    str_total.short_description = _('Total')


class CustomerQuotationRowAdmin(QuotationRowAdmin):
    pass


class SupplierQuotationRowAdmin(QuotationRowAdmin):
    pass


class QuotationAdminForm(AdminModelForm):
    class Meta:
        model = Quotation

    # TODO: FIXME - occorre filtrare queryset degli oggetti in ADD e EDIT

class CusQuotationAdminForm(QuotationAdminForm):
    owner = forms.ModelChoiceField(
        Organization.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('CustomerOrganization',)
    )
    recipient = forms.ModelChoiceField(
        Organization.objects.all(), required=True,
        widget=autocomplete_light.ChoiceWidget('CustomerOrganization',)
    )

class SupQuotationAdminForm(QuotationAdminForm):
    owner = forms.ModelChoiceField(
        Organization.objects.filter(organization_type__category="sup"), required=True,
        widget=autocomplete_light.ChoiceWidget('SupplierOrganization',)
    )
    recipient = forms.ModelChoiceField(
        Organization.objects.filter(organization_type__category="sup"), required=True,
        widget=autocomplete_light.ChoiceWidget('SupplierOrganization',)
    )

def delete_selected(modeladmin, request, queryset):
    #controllo dei permessi
    opts = modeladmin.model._meta
    app_label = opts.app_label
    if not modeladmin.has_delete_permission(request):
        raise PermissionDenied
    using = router.db_for_write(modeladmin.model)
    deletable_objects, perms_needed, protected = get_deleted_objects(
        queryset, opts, request.user, modeladmin.admin_site, using)

    if request.POST.get('post'):
        if perms_needed:
            raise PermissionDenied
        n = queryset.count()
        if n:
            for obj in queryset:
                obj_display = force_text(obj)
                modeladmin.log_deletion(request, obj, obj_display)
            queryset.delete()
            modeladmin.message_user(request, _("Successfully deleted %(count)d %(items)s.") % {
                "count": n, "items": model_ngettext(modeladmin.opts, n)
            }, messages.SUCCESS)
        return None

    qs_accepted = len(queryset.filter(Q(accepted=True) | Q(order_done=True) | Q(submitted=True)))
    if qs_accepted > 0:
        messages.error(request, _("Quotation already ordered. It's not deletable"))
        return None
    return delete_selected_(modeladmin, request, queryset)


delete_selected.short_description = ugettext_lazy("Delete selected %(verbose_name_plural)s")


class QuotationAdmin(JumboModelAdmin):
    class Media:
        js = ("quotation/js/admin_dajaxice.js",)

    fieldsets = (
        (_('logs admin'), {'fields': (
            'creator', 'date_create',
            'last_modifier', 'date_last_modify',
        ), 'classes': ('collapse',)}),
        (_('main'), {
            'fields': (
                'status',
                ('date_q',),
                ('number', 'revision', 'revision_year'),
                ('owner', 'recipient',),
                'office',
                ('contact', 'project',),
                'payment_form',
                'store_cause',
                'currency',
                'vat',
                'price_list',
#                'commision',
#                ('vector', 'delivery',),
                'delivery',
                ('date_inspected', 'inspected'),
                ('date_submitted', 'submitted'),
                ('date_accepted', 'accepted'),
                ('date_delivery', 'shipping_address',),
                ('delivery_window', 'quote_firm_for',),
                'description',
            )
        }),
        (_('note'), {'fields': ('note',), 'classes': ('collapse',)}),
    )
    list_display = (
        'get_date_q', 'get_number', 'get_code', 'get_detail', 'get_total',
        'send_email', 'create_odt', 'create_pdf', 'inspected', 'submitted', 'accepted',
#        'procedure', 'get_document_icon', 'get_detail_icon', 'get_edit_icon',
        'order_done', 'get_detail_icon', 'get_edit_icon',
    )
    list_display_csv = (
        'get_date_q', 'get_number', 'owner', 'contact', 'description',
        'payment_form', 'store_cause',
#        'get_income', 'get_fee', 'get_discount', 'get_vat', 'get_total'
    )
    advanced_search_fields = (
        ('owner__name__icontains', 'owner__code'),
        ('recipient__name__icontains', 'recipient__code'),
        ('payment_form__in', 'store_cause__in'),
        ('date_q__year', 'date_q__range'),
        ('date_q__gte', 'date_q__lte',),
        ('number', 'revision',),
        ('inspected', 'submitted',),
        ('accepted', 'order_done',),
    )

    tabs = (
        ('main', {}),
        ('rows', {'items': [QuotationRowAjaxInline]}),
    )

    form = QuotationAdminForm

    def get_date_q(self, obj):
        return obj.date_q.strftime("%d/%m/%Y")

    get_date_q.short_description = _('Date')
    get_date_q.admin_order_field = 'date_q'

    def get_payment_form(self, obj):
        return """<span title="%s">%s</span>""" % (obj.payment_form.name, obj.payment_form.code)

    get_payment_form.short_description = _("Payment Form")
    get_payment_form.allow_tags = True
    get_payment_form.admin_order_field = 'payment_form__code'

    def get_store_cause(self, obj):
        return """<span title="%s">%s</span>""" % (obj.store_cause.name, obj.store_cause.code)

    get_store_cause.short_description = _("Store Cause")
    get_store_cause.allow_tags = True
    get_store_cause.admin_order_field = 'store_cause__code'

    def get_income(self, obj):
        return obj.totals['income']

    get_income.short_description = _('Total Income')

    def get_fee(self, obj):
        return obj.totals['fee']

    get_fee.short_description = _('Total Fee')

    def get_discount(self, obj):
        return obj.totals['discount']

    get_discount.short_description = _('Total Discount')

    def get_vat(self, obj):
        return obj.totals['vat']

    get_vat.short_description = _('Total VAT')

    def get_total(self, obj):
        return obj.totals['total']

    get_total.short_description = _('Total')

    def get_code(self, obj):
        return obj.owner.code

    get_code.short_description = _('Code')

    def get_detail(self, obj):
        owner = """<a href=%(reverse_url)s>%(owner)s</a>""" % {
            'reverse_url': reverse('admin:organization_organization_change', args=(obj.owner.id,)),
            'owner': obj.owner.name
        }
        contact = obj.contact or ""
        description = obj.description
        return "<b>%s</b><br/>%s<br/>%s" % (owner, contact, description)

    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'owner'

    def send_email(self, obj):
#        return """
#            <a href="%(href)s" title="%(title)s">
#                <img src='%(url)sadmin/images/email.png' alt='%(title)s' title='%(title)s'/>
#            </a>
#        """ % {
#            'href' : reverse('admin:quotation_quotation_send_email', obj.id),
#            'url':settings.MEDIA_URL,
#            'title': ugt("Send Email")
#        }
        return ""

    send_email.short_description = _("Email")
    send_email.allow_tags = True

    def create_odt(self, obj):
#        return """
#            <a href="%(href)s" title="%(title)s">
#                <img src='%(url)sadmin/images/writer.gif' alt='%(title)s' title='%(title)s'/>
#            </a>
#        """ % {
#            'href' : reverse('admin:quotation_quotation_create_odt', obj.id),
#            'url':settings.MEDIA_URL,
#            'title': ugt("Create Quotation ODT"),
#        }
        return ""

    create_odt.short_description = _("ODT")
    create_odt.allow_tags = True

    def create_pdf(self, obj):
#        return """
#            <a href="%(href)s" title="%(title)s">
#                <img src='%(url)sadmin/images/pdf_icon.png' alt='%(title)s' title='%(title)s'/>
#            </a>
#        """ % {
#            'href' :reverse('admin:quotation_quotation_create_pdf', obj.id),
#            'url':settings.MEDIA_URL,
#            'title': ugt("Create Quotation PDF")
#        }
        return ""

    create_pdf.short_description = _("PDF")
    create_pdf.allow_tags = True

    def clone_quotation(self, request, queryset):
        """Clone quotation"""
        rows_updated = 0
        for obj in queryset:
            try:
                self.clone(request, obj)
                rows_updated += 1
            except Exception, e:
                messages.error(request, "%s: %s" % (obj.get_number, e))
        if rows_updated == 0:
            messages.error(request, _("ATTENTION! Nothing was cloned"))
        elif rows_updated == 1:
            message_bit = _("1 object was")
            messages.info(request, _("%s successfully cloned.") % message_bit)
        else:
            message_bit = _("%s objects were") % rows_updated
            messages.info(request, _("%s successfully cloned.") % message_bit)

    clone_quotation.short_description = _("Clone Quotation")

    def set_inspected(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            try:
                row.inspected = True
                row.date_inspected = datetime.date.today()
                row.save()
                rows_updated += 1
            except Exception, e:
                msg = "%s - %s -  %s" % ('e', row.number, row.date_q)
                messages.error(request, msg)
        if rows_updated == 1:
            message_bit = "1 Quotation was"
        else:
            message_bit = "%s Quotation were" % rows_updated
        messages.info(request, "%s successfully inspected." % message_bit)

    set_inspected.short_description = _("Set Quotations as inspected")

    def set_accepted(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            try:
                row.accepted = True
                row.date_accepted = datetime.date.today()
                row.save()
                rows_updated += 1
            except Exception, e:
                msg = "%s - %s -  %s" % ('e', row.number, row.date_q)
                messages.error(request, msg)
        if rows_updated == 1:
            message_bit = "1 Quotation was"
        else:
            message_bit = "%s Quotation were" % rows_updated
        messages.info(request, "%s successfully accepted." % message_bit)

    set_accepted.short_description = _("Set Quotations as accepted")

    def set_submitted(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            try:
                row.submitted = True
                row.date_submitted = datetime.date.today()
                row.save()
                rows_updated += 1
            except Exception, e:
                msg = "{} - {} - {}".format('e', row.number, row.date_q)
                messages.error(request, msg)
        if rows_updated == 1:
            message_bit = "1 Quotation was"
        else:
            message_bit = "%s Quotation were" % rows_updated
        messages.info(request, "%s successfully submitted." % message_bit)

    set_submitted.short_description = _("Set Quotations as submitted")

    def get_actions(self, request):
        actions = super(QuotationAdmin, self).get_actions(request)
        del actions['set_disactive']
        del actions['set_active']
        return actions

    actions = [set_inspected, set_accepted, set_submitted, delete_selected]


class CustomerQuotationAdmin(QuotationAdmin):
    form = CusQuotationAdminForm


class SupplierQuotationAdmin(QuotationAdmin):
    form = SupQuotationAdminForm



admin.site.register(Quotation, QuotationAdmin)
admin.site.register(CustomerQuotation, CustomerQuotationAdmin)
admin.site.register(SupplierQuotation, CustomerQuotationAdmin)

admin.site.register(QuotationRow, QuotationRowAdmin)
admin.site.register(CustomerQuotationRow, CustomerQuotationRowAdmin)
admin.site.register(SupplierQuotationRow, SupplierQuotationRowAdmin)

admin.site.register(VatRow)
admin.site.register(CustomerVatRow)
admin.site.register(SupplierVatRow)

register_inline(QuotationRowAjaxInline)
