# -*- coding: utf-8 -*-
"""
.. _models:

models
=======

Modello della Quotation, QuotationProject, Quotation Row

Deps: Organization, Contact, OrganizationAddress
    Currency, Delivery, Vat, OrganizationPriceList
    Product, PaymentForm, Office
    StoreCause, Project

.. autoclass:: Quotation
   :members:

.. autoclass:: QuotationRow
   :members:

.. autoclass:: QuotationProject
   :members:

"""
import decimal

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.i18n.models import TranslationModel, I18NModel
from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE

from jmb.organization.models import Organization, Contact, OrganizationAddress
from jmb.currency.models import Currency
from jmb.carriage.models import Delivery
from jmb.vat.models import Vat
from jmb.price.models import OrganizationPriceList
from jmb.product.models import Product
from jmb.payment.models import PaymentForm
from jmb.office.models import Office
from jmb.store.models import StoreCause
from jmb.project.models import Project


class QuotationProject(Project):
    class Meta:
        verbose_name = _("Project")
        verbose_name_plural = _("Projects")

    def __unicode__(self):
        return unicode(self.name)

    @classmethod
    def create_from(cls, project):
        """
        Crea una QuotationProject a partire da una jmb.project.Project
        """
        new = cls(project_ptr=project)
        new.__dict__.update(project.__dict__)
        new.save()
        return new


class CustomerQuotationManager(models.Manager):
    def get_queryset(self):
        return super(CustomerQuotationManager, self).get_queryset().filter(
            store_cause__category='cus'
        )


class SupplierQuotationManager(models.Manager):
    def get_queryset(self):
        return super(SupplierQuotationManager, self).get_queryset().filter(
            store_cause__category='sup'
        )


class Quotation(DateModel, UserModel):
    owner = models.ForeignKey(
        Organization,
        related_name="owner_quotations",
        db_index=True,
        verbose_name=_("Bill to"),
    )
    recipient = models.ForeignKey(
        Organization,
        null=True, blank=True,
        related_name="recipient_quotations",
        db_index=True,
        verbose_name=_("Ship to"),
    )
    project = models.ForeignKey(
        QuotationProject,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Project"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Payment Form"),
    )
    office = models.ForeignKey(
        Office,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Office"),
    )
    contact = models.ForeignKey(
        Contact,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Contact"),
    )
    currency = models.ForeignKey(
        Currency,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Currency")
    )
    delivery = models.ForeignKey(
        Delivery,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Delivery")
    )
    vat = models.ForeignKey(
        Vat,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Vat")
    )
    #    agent = models.ForeignKey (
    #        Agent,
    #        null = True, blank = True,
    #        related_name = "quotations",
    #        db_index = True,
    #        verbose_name = _("Agent"),
    #    )
    #    commision = models.ForeignKey (
    #        Commision,
    #        blank = True, null = True,
    #        related_name = "quotations",
    #        db_index = True,
    #        verbose_name = _("Agent & Commision")
    #    )
    price_list = models.ForeignKey(
        OrganizationPriceList,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Prices List & Discount")
    )
    #    vector = models.ForeignKey (
    #        Vector,
    #        blank = True, null = True,
    #        related_name = "quotations",
    #        db_index = True,
    #        verbose_name = _("Vector")
    #    )
    store_cause = models.ForeignKey(
        StoreCause,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Store Cause")
    )
    shipping_address = models.ForeignKey(
        OrganizationAddress,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Shipping Address")
    )
    description = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_("Description")
    )
    full_description = HTMLField(
        blank=True,
        null=True,
        verbose_name=_("Full description"),
    )
    number = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Number"),
        db_column="qnumber"
    )
    revision = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Revision"),
    )
    revision_year = models.IntegerField(
        verbose_name=_("First revision year")
    )
    date_q = models.DateField(
        db_index=True,
        verbose_name=_("Date")
    )
    '''
    preventivo già trasformato in procedure
    '''
    order_done = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Ordered"),
    )
    '''
    data prevista consegna
    '''
    date_delivery = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Date Delivery"),
    )
    '''
    campi condizioni generali
    validita offerta
    '''
    quote_firm_for = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Quote Firm For"),
    )
    '''
    tempi di consegna
    '''
    delivery_window = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Delivery Window"),
    )
    '''
    fatto sopralluogo
    '''
    inspected = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Inspected"),
    )
    '''
    data sopralluogo
    '''
    date_inspected = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Inspected Date"),
    )
    '''
    preventivo presentato
    '''
    submitted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Submitted"),
    )
    '''
    data presentazione
    '''
    date_submitted = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Submitted Date"),
    )
    '''
    preventivo accettato
    '''
    accepted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Accepted"),
    )
    '''
    data accettazione
    '''
    date_accepted = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Accepted Date"),
    )
    total_income = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Income"),
    )
    total_fee = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Fee"),
    )
    total_discount = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Discount"),
    )
    total_vat = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total VAT"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )

    def can_be_delete(self):
        if self.order_done or self.accepted or self.submitted:
            return False
        return True

    def get_number(self):
        number = ""
        if self.number:
            number = str(self.number).rjust(settings.QUOTATION_ZEROFILL_NUMBER, '0')
        if self.revision == 0:
            return "%s-%s" % (number, self.revision_year)
        return "%s/%s-%s" % (number, self.revision, self.revision_year)

    get_number.short_description = _("Number")

    def get_totals(self):
        totals = dict()
        totals['income'] = decimal.Decimal(self.total_income)
        totals['fee'] = decimal.Decimal(self.total_fee)
        totals['discount'] = decimal.Decimal(self.total_discount)
        totals['vat'] = decimal.Decimal(self.total_vat)
        totals['total_no_vat'] = (
            decimal.Decimal(self.total_income) +
            decimal.Decimal(self.total_fee) -
            decimal.Decimal(self.total_discount)
        )
        totals['total'] = totals['total_no_vat'] + decimal.Decimal(self.total_vat)
        return totals

    totals = property(get_totals)

    class Meta:
        verbose_name = _("quotation")
        verbose_name_plural = _("quotations")
        permissions = (
            ('add_customer_quotation', 'Can ADD Customer quotation'),
            ('add_supplier_quotation', 'Can ADD Supplier quotation'),

            ('change_customer_quotation', 'Can CHANGE Customer quotation'),
            ('change_supplier_quotation', 'Can CHANGE Supplier quotation'),

            ('view_quotation', 'Can VIEW quotation'),
            ('view_customer_quotation', 'Can VIEW Customer quotation'),
            ('view_supplier_quotation', 'Can VIEW Supplier quotation'),

            ('list_quotation', 'Can LIST quotation'),
            ('list_customer_quotation', 'Can LIST Customer quotation'),
            ('list_supplier_quotation', 'Can LIST Supplier quotation'),
        )

    def __unicode__(self):
        return u"%(date)s, %(number)s" % {'date': self.date_q, 'number': self.number}


class CustomerQuotation(Quotation):
    objects = CustomerQuotationManager()

    class Meta:
        proxy = True
        verbose_name = _("customer quotation")
        verbose_name_plural = _("customer quotations")
        permissions = (
            ('add_customer_quotation', 'Can ADD Customer quotation'),
            ('change_customer_quotation', 'Can CHANGE Customer quotation'),
            ('view_customer_quotation', 'Can VIEW Customer quotation'),
            ('list_customer_quotation', 'Can LIST Customer quotation'),
        )


class SupplierQuotation(Quotation):
    objects = SupplierQuotationManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier quotation")
        verbose_name_plural = _("supplier quotations")
        permissions = (
            ('add_supplier_quotation', 'Can ADD Supplier quotation'),
            ('change_supplier_quotation', 'Can CHANGE Supplier quotation'),
            ('view_supplier_quotation', 'Can VIEW Supplier quotation'),
            ('list_supplier_quotation', 'Can LIST Supplier quotation'),
        )


class CustomerQuotationRowManager(models.Manager):
    def get_queryset(self):
        return super(CustomerQuotationRowManager, self).get_queryset().filter(
            store_cause__category='cus'
        )


class SupplierQuotationRowManager(models.Manager):
    def get_queryset(self):
        return super(SupplierQuotationRowManager, self).get_queryset().filter(
            store_cause__category='cus'
        )


class QuotationRow(DateModel, UserModel, OrderedModel):
    quotation = models.ForeignKey(
        Quotation,
        related_name="rows",
        db_index=True,
        verbose_name=_("Quotation"),
    )
    ### da aggiungere anche anno
    product = models.ForeignKey (
        Product,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Product"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Vat"),
    )
    store_cause = models.ForeignKey(
        StoreCause,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Store Cause"),
    )
    project = models.ForeignKey(
        QuotationProject,
        null=True, blank=True,
        db_index=True,
        related_name="quotationrows",
        verbose_name=_("Project"),
    )
    quantity = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Quantity"),
    )
    m_unit = models.CharField(
        max_length=2,
        choices=settings.PRODUCT_PRODUCT_UNIT_TYPE,
        default="pz",
        db_index=True,
        verbose_name=_("Unit"),
    )
    price = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Price"),
    )
    # costo manodopera
    labor_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Labor cost"),
    )
    # costo materiale
    material_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Matrial cost"),
    )
    # costo dei servizi - noleggio e trasporti
    services_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Services cost"),
    )
    # ricarico da applicare sui precedenti costi per determinare il prezzo
    # di vendita
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )
    fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Fee"),
    )
    discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount"),
    )
    description = models.CharField(
        max_length=200,
        blank=True, null=True,
        verbose_name=_("Description")
    )
    long_description = models.TextField(
        blank=True, null=True,
        verbose_name=_("Long Description"),
    )
    to_print = models.BooleanField(
        db_index=True,
        default=True,
        verbose_name=_("To Print"),
    )
    # riepilogo
    # campo booleano utilizzato per inserire la riga nel report di riepilogo
    summary = models.BooleanField(
        db_index=True,
        default=True,
        verbose_name=_("Summary"),
    )

    class Meta:
        verbose_name = _("quotation row")
        verbose_name_plural = _("quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']
        permissions = (
            ('add_customer_quotationrow', 'Can ADD Customer quotation row'),
            ('add_supplier_quotationrow', 'Can ADD Supplier quotation row'),

            ('change_customer_quotationrow', 'Can CHANGE Customer quotation row'),
            ('change_supplier_quotationrow', 'Can CHANGE Supplier quotation row'),

            ('view_quotationrow', 'Can VIEW quotation row'),
            ('view_customer_quotationrow', 'Can VIEW Customer quotation row'),
            ('view_supplier_quotationrow', 'Can VIEW Supplier quotation row'),

            ('list_quotationrow', 'Can LIST quotation row'),
            ('list_customer_quotationrow', 'Can LIST Customer quotation row'),
            ('list_supplier_quotationrow', 'Can LIST Supplier quotation row'),
        )

    def __unicode__(self):
        return u"%(quotation)s, %(product)s" % {
            'quotation': self.quotation, 'product': self.product
        }


class CustomerQuotationRow(QuotationRow):
    objects = CustomerQuotationRowManager()

    class Meta:
        proxy = True
        verbose_name = _("customer quotation row")
        verbose_name_plural = _("customer quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']
        permissions = (
            ('add_customer_quotationrow', 'Can ADD Customer quotation row'),
            ('change_customer_quotationrow', 'Can CHANGE Customer quotation row'),
            ('view_customer_quotationrow', 'Can VIEW Customer quotation row'),
            ('list_customer_quotationrow', 'Can LIST Customer quotation row'),
        )


class SupplierQuotationRow(QuotationRow):
    objects = SupplierQuotationRowManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier quotation row")
        verbose_name_plural = _("supplier quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']
        permissions = (
            ('add_supplier_quotationrow', 'Can ADD Supplier quotation row'),
            ('change_supplier_quotationrow', 'Can CHANGE Supplier quotation row'),
            ('view_supplier_quotationrow', 'Can VIEW Supplier quotation row'),
            ('list_supplier_quotationrow', 'Can LIST Supplier quotation row'),
        )


class CustomerVatRowManager(models.Manager):
    def get_queryset(self):
        return super(CustomerVatRowManager, self).get_queryset().filter(
            quotation__store_cause__category='cus'
        )


class SupplierVatRowManager(models.Manager):
    def get_queryset(self):
        return super(SupplierVatRowManager, self).get_queryset().filter(
            quotation__store_cause__category='sup'
        )


class VatRow(DateModel, UserModel, StatusModel):
    quotation = models.ForeignKey(
        Quotation,
        related_name="vattotalquotationrows",
        db_index=True,
        verbose_name=_("Quotation"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name="vattotalquotationrows",
        db_index=True,
        verbose_name=_("Vat"),
    )
    total_income = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Income"),
    )
    total_fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Fee"),
    )
    total_discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Discount"),
    )
    total_vat = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Vat"),
    )

    class Meta:
        verbose_name = _("vat row")
        verbose_name_plural = _("vat rows")
        ordering = ('vat__ordering', 'vat__code')
        permissions = (
            ('add_customer_vatrow', 'Can ADD Customer quotation row'),
            ('add_supplier_vatrow', 'Can ADD Supplier quotation row'),

            ('change_customer_vatrow', 'Can CHANGE Customer quotation row'),
            ('change_supplier_vatrow', 'Can CHANGE Supplier quotation row'),

            ('view_vatrow', 'Can VIEW quotation row'),
            ('view_customer_vatrow', 'Can VIEW Customer quotation row'),
            ('view_supplier_vatrow', 'Can VIEW Supplier quotation row'),

            ('list_customer_vatrow', 'Can LIST Customer quotation row'),
            ('list_supplier_vatrow', 'Can LIST Supplier quotation row'),
        )

        def __unicode__(self):
            return u"%s %s %s %s" % (
                self.quotation, self.vat, self.total_income, self.total_vat
            )


class CustomerVatRow(VatRow):
    objects = CustomerVatRowManager()

    class Meta:
        proxy = True
        verbose_name = _("customer vat row")
        verbose_name_plural = _("customer vat rows")
        ordering = ('vat__ordering', 'vat__code')
        permissions = (
            ('add_customer_vatrow', 'Can ADD Customer quotation row'),
            ('change_customer_vatrow', 'Can CHANGE Customer quotation row'),
            ('view_customer_vatrow', 'Can VIEW Customer quotation row'),
            ('list_customer_vatrow', 'Can LIST Customer quotation row'),
        )


class SupplierVatRow(VatRow):
    objects = SupplierVatRowManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier vat row")
        verbose_name_plural = _("supplier vat rows")
        ordering = ('vat__ordering', 'vat__code')
        permissions = (
            ('add_supplier_vatrow', 'Can ADD Supplier quotation row'),
            ('change_supplier_vatrow', 'Can CHANGE Supplier quotation row'),
            ('view_supplier_vatrow', 'Can VIEW Supplier quotation row'),
            ('list_supplier_vatrow', 'Can LIST Supplier quotation row'),
        )
