# -*- coding: utf-8 -*-
import datetime
from django.conf import settings
from django.db import models, transaction
from django.utils.translation import ugettext_lazy as _
from django.template import Context
from django.template.loader import render_to_string

from jmb.core.db.models import DateModel, UserModel, OrderedModel
from jmb.core.db.fields import HTMLField
from jmb.core.utils.functions import zerofill
from django.core.mail import EmailMultiAlternatives

from jmb.organization.models import Organization, Contact, OrganizationAddress
from jmb.currency.models import Currency
from jmb.carriage.models import Delivery
from jmb.vat.models import Vat
from jmb.payment.models import PaymentForm
from jmb.office.models import Office
from jmb.store.models import StoreCause

from jmb.price.models import OrganizationPriceList
from jmb.product.models import Product

from dfk import DeferredForeignKey

from jmb.quotation.procedure_methods import ProcedureMethods, ProcedureRowMethods


class AbstractQuotation(DateModel, UserModel, ProcedureMethods):
    owner = models.ForeignKey(
        Organization,
        related_name="owner_quotations",
        db_index=True,
        verbose_name=_("Bill to"),
    )
    recipient = models.ForeignKey(
        Organization,
        related_name="recipient_quotations",
        db_index=True,
        verbose_name=_("Ship to"),
    )
    project = models.ForeignKey(
        'quotation.QuotationProject',
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Project"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Payment Form"),
    )
    office = models.ForeignKey(
        Office,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Office"),
    )
    contact = models.ForeignKey(
        Contact,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Contact"),
    )
    currency = models.ForeignKey(
        Currency,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Currency")
    )
    delivery = models.ForeignKey(
        Delivery,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Delivery")
    )
    vat = models.ForeignKey(
        Vat,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Vat")
    )
    # agent = models.ForeignKey(
    #     Agent,
    #     blank=True, null=True,
    #     related_name="quotations",
    #     db_index=True,
    #     verbose_name=_("agent")
    # )
    # vector = models.ForeignKey(
    #     Agent,
    #     blank=True, null=True,
    #     related_name="quotations",
    #     db_index=True,
    #     verbose_name=_("agent")
    # )
    price_list = models.ForeignKey(
        OrganizationPriceList,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Prices List & Discount")
    )
    store_cause = models.ForeignKey(
        StoreCause,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Store Cause")
    )
    shipping_address = models.ForeignKey(
        OrganizationAddress,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Shipping Address")
    )
    description = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_("Description")
    )
    full_description = HTMLField(
        blank=True,
        null=True,
        verbose_name=_("Full description"),
    )
    number = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Number"),
        db_column="qnumber"
    )
    revision = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Revision"),
    )
    revision_year = models.IntegerField(
        verbose_name=_("First revision year")
    )
    date_q = models.DateField(
        db_index=True,
        verbose_name=_("Date")
    )
    order_done = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Ordered"),
    )
    date_delivery = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Date Delivery"),
    )
    quote_firm_for = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Quote Firm For"),
    )
    delivery_window = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Delivery Window"),
    )
    inspected = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Inspected"),
    )
    date_inspected = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Inspected Date"),
    )
    submitted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Submitted"),
    )
    date_submitted = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Submitted Date"),
    )
    accepted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Accepted"),
    )
    date_accepted = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Accepted Date"),
    )
    total_income = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Income"),
    )
    total_fee = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Fee"),
    )
    total_discount = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Discount"),
    )
    total_vat = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total VAT"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )

    class Meta:
        abstract = True
        verbose_name = _("quotation")
        verbose_name_plural = _("quotations")

    def __unicode__(self):
        return u"%(date)s, %(number)s" % {'date': self.date_q, 'number': self.number}

    def save(self, *args, **kwargs):
        if not self.number:
            # number
            from jmb.store.utils import create_update_storedocumentnumber
            self.number = create_update_storedocumentnumber(
                'q', curr_date=self.date_q, store_document=self.store_cause.store_document
            )
        if not self.revision:
            self.revision = 0

        if not self.revision_year:
            self.revision_year = datetime.date.today().year

        # date inspected
        if self.date_inspected and not self.inspected:
            self.inspected = True
        if not self.date_inspected:
            self.inspected = False

        # date accepted
        if self.date_accepted and not self.accepted:
            self.accepted = True
        if not self.date_accepted:
            self.accepted = False

        # date submitted
        if self.date_submitted and not self.submitted:
            self.submitted = True
        if not self.date_submitted:
            self.submitted = False

        super(AbstractQuotation, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if (
            not self.procedure_done and
            not self.accepted and
            not self.submitted
        ):
            return super(AbstractQuotation, self).delete()

    # TODO : Verificare prima metodo clone di Luca
    @transaction.commit_manually
    def clone(self):
        try:
            new_kwargs = dict([
                (fld.name, getattr(self, fld.name)) for fld in self._meta.fields if fld.name != 'id'
            ])
            # return self.__class__.objects.create(**new_kwargs)
            new_obj = self.__class__(**new_kwargs)
            new_obj.save()
            transaction.commit()
        except Exception, e:
            transaction.rollback()
            raise Exception(e)

    # def clone(self, request, obj):
    #     try:
    #         old_obj = Quotation.objects.get(pk=obj.pk)
    #         obj.pk = None
    #         obj.recipient = None
    #         obj.procedure = False
    #         obj.payment_form = None
    #         obj.office = None
    #         obj.contact = None
    #         obj.currency = None
    #         obj.delivery = None
    #         obj.vat = None
    #         obj.agent = None
    #         obj.commision = None
    #         obj.price_list = None
    #         obj.vector = None
    #         obj.shipping_address = None
    #         obj.number = None
    #         obj.revision = None
    #         obj.date_q = datetime.date.today()
    #         obj.inspected = None
    #         obj.date_inspected = None
    #         obj.date_delivery = None
    #         obj.submitted = False
    #         obj.date_submitted = None
    #         obj.accepted = False
    #         obj.date_accepted = None
    #         obj.note = ""
    #         obj.save(force_insert=True)
    #
    #         ## salvo many to many collegate
    #         for m2m in obj._meta.many_to_many:
    #             setattr(obj, m2m.name, getattr(old_obj, m2m.name).all())
    #
    #         # salvo related
    #         for row in old_obj.rows.all():
    #             row.pk = None
    #             row.project = None
    #             obj.rows.add(row)
    #
    #         obj.save()
    #         transaction.commit()
    #         return ""
    #     except Exception, e:
    #         transaction.rollback()
    #         raise Exception(e)

    @transaction.commit_manually
    def do_revision(self, request):
        try:
            old_obj = self
            self.pk = None
            self.procedure_done = False
            self.date_delivery = None
            self.submitted = False
            self.date_submitted = None
            self.accepted = False
            self.date_accepted = None

            self.date_q = datetime.date.today()
            # controllo revisioni precedenti con:
            # - stesso numero
            # - stesso anno di prima revisione
            # - stesso intestatario
            revision = self.__class__.objects.filter(
                number=self.number, revision_year=self.revision_year,
                owner=self.owner
            ).count()
            self.revision = revision + 1
            self.save(force_insert=True)

            # salvo many to many collegate
            for m2m in self._meta.many_to_many:
                setattr(self, m2m.name, getattr(old_obj, m2m.name).all())

            # salvo related
            for row in old_obj.rows.all():
                row.pk = None
                self.rows.add(row)

            self.save()
            transaction.commit()
            return ""
        except Exception, e:
            transaction.rollback()
            raise Exception(e)

    def update_totals(self):
        try:
            first_quotationrow = self.rows.all()[0]
            first_quotationrow.update_totals()
        except Exception:
            pass

    def can_be_deleted(self):
        # if self.order_done or self.accepted or self.submitted:
        #     return False
        return True

    def can_be_modified(self):
        return True

    def can_be_cloned(self):
        return True

    def can_be_revisioned(self):
        return True

    def get_number(self):
        number = ""
        if self.number:
            # number = zerofill(self.number, settings.QUOTATION_QUOTATION_NUMBER_ZEROFILL)
            number = zerofill(self.number, 3)
        if self.revision == 0:
            return "%s-%s" % (number, self.revision_year)
        return "%s/%s-%s" % (number, self.revision, self.revision_year)
    get_number.short_description = _("Number")

    def get_total(self):
        return self.total_income - self.total_discount + self.total_fee + self.total_vat

    def get_totals(self):
        totals = dict()
        totals['income'] = self.total_income
        totals['fee'] = self.total_fee
        totals['discount'] = self.total_discount
        totals['vat'] = self.total_vat
        totals['total_no_vat'] = (
            self.total_income + self.total_fee - self.total_discount
        )
        totals['total'] = totals['total_no_vat'] + self.total_vat
        return totals
    totals = property(get_totals)

    def default_invoice_address(self):
        """
        restituisce il primo indirizzo di fatturazione
        """
        invoice_addresses = self.owner.get_addresses().filter(
            address_scopes__is_invoice_address=True)
        if invoice_addresses:
            return invoice_addresses[0]
        return None

    def default_invoice_address_dict(self):
        """
        restituisce un dizionario con le prime 3 righe dell'indirizzo di fatturazione
        """
        address = self.default_invoice_address()
        address_dict = dict()
        address_dict["row1"] = "{} {}, {}".format(
            address.get_address_type_display() or "", address.name, address.number)
        address_dict["row2"] = "{} {} ({})".format(
            address.zip_code, address.locality, address.province)
        address_dict["row3"] = "{}".format(address.country)
        return address_dict

    def shipping_address_dict(self):
        address_dict = dict()
        address_dict["row1"] = "{} {}, {}" % (
            self.shipping_address.get_address_type_display() or "",
            self.shipping_address.name,
            self.shipping_address.number
        )
        address_dict["row2"] = "{} {} ({})" % (
            self.shipping_address.zip_code,
            self.shipping_address.locality,
            self.shipping_address.province
        )
        address_dict["row3"] = "{}" % self.shipping_address.country
        return address_dict

    def send_email(self, data_dict,):
        try:
            # static_path = data_dict["static_path"]
            email_template = data_dict["template"]
            sent_from = data_dict["sent_from"]
            contact = self.contact if self.contact else None
        except Exception:
            return False
        ctx_dict = Context({
            'form': data_dict,
            'contact': contact,
            'sent_from': sent_from,
            # 'static': static_path,
        })
        html_message = render_to_string(email_template, ctx_dict)
        from_email = settings.QUOTATION_QUOTATION_SEND_EMAIL_FROM_EMAIL
        email_to = []
        subject = data_dict['subject']
        emails = [c.email for c in data_dict['contacts']]
        other_emails = data_dict['other_emails']
        email_to.extend(emails)
        email_to.extend(other_emails)
        msg = EmailMultiAlternatives(subject, "", from_email, email_to)
        msg.attach_alternative(html_message, "text/html")
        try:
            msg.send()
        except Exception, e:
            return False, _('Alert : email not sent. %s' % e)
        return True, _('Email sent correctly')


class AbstractQuotationRow(DateModel, UserModel, OrderedModel, ProcedureRowMethods):
    # quotation = models.ForeignKey(
    #     Quotation,
    #     related_name="rows",
    #     db_index=True,
    #     verbose_name=_("Quotation"),
    # )
    quotation = DeferredForeignKey(
        related_name="rows",
        db_index=True,
        verbose_name=_("Quotation"),
    )
    product = models.ForeignKey(
        Product,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Product"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Vat"),
    )
    store_cause = models.ForeignKey(
        StoreCause,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Store Cause"),
    )
    project = models.ForeignKey(
        'quotation.QuotationProject',
        null=True, blank=True,
        db_index=True,
        related_name="quotationrows",
        verbose_name=_("Project"),
    )
    quantity = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Quantity"),
    )
    m_unit = models.CharField(
        max_length=2,
        choices=settings.PRODUCT_PRODUCT_UNIT_TYPE,
        default="pz",
        db_index=True,
        verbose_name=_("Unit"),
    )
    price = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Price"),
    )
    labor_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Labor cost"),
    )
    material_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Matrial cost"),
    )
    services_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Services cost"),
    )
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )
    fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Fee"),
    )
    discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount"),
    )
    description = models.CharField(
        max_length=200,
        blank=True, null=True,
        verbose_name=_("Description")
    )
    long_description = models.TextField(
        blank=True, null=True,
        verbose_name=_("Long Description"),
    )
    to_print = models.BooleanField(
        db_index=True,
        default=True,
        verbose_name=_("To Print"),
    )
    # riepilogo
    # campo booleano utilizzato per inserire la riga nel report di riepilogo
    summary = models.BooleanField(
        db_index=True,
        default=True,
        verbose_name=_("Summary"),
    )

    class Meta:
        abstract = True
        verbose_name = _("quotation row")
        verbose_name_plural = _("quotation rows")

    def __unicode__(self):
        return u"{quotation)}, {product}".format(
            {'quotation': self.quotation, 'product': self.product})

    def save(self, force_insert=False, force_update=False):
        super(AbstractQuotationRow, self).save(
            force_insert=force_insert, force_update=force_update)
        self.update_totals()

    def save_not_update(self, force_insert=False, force_update=False):
        super(AbstractQuotationRow, self).save(
            force_insert=force_insert, force_update=force_update)

    def update_totals(self):
        from jmb.quotation.models import VatRow
        quotation = self.quotation
        rows = self.quotation.rows.all()
        VatRow.objects.filter(quotation=quotation).delete()
        total = {'income': 0, 'fee': 0, 'discount': 0, 'total': 0, 'vat': 0}
        subtotal = {'income': {}, 'fee': {}, 'discount': {}, 'vat': {}, 'total': {}}
        for row in rows:
            subtotal['income'][row.vat] = subtotal['income'].get(row.vat, 0)
            subtotal['income'][row.vat] += row.total_income

            subtotal['fee'][row.vat] = subtotal['fee'].get(row.vat, 0)
            subtotal['fee'][row.vat] += row.total_fee
            subtotal['discount'][row.vat] = subtotal['discount'].get(row.vat, 0)
            subtotal['discount'][row.vat] += row.total_discount

            subtotal['total'][row.vat] = subtotal['total'].get(row.vat, 0)
            subtotal['total'][row.vat] += row.total

        for vat in subtotal['income'].keys():
            # calcolo Totali Procedura
            total['income'] += subtotal['income'][vat]
            total['fee'] += subtotal['fee'][vat]
            total['discount'] += subtotal['discount'][vat]
            total['total'] += subtotal['total'][vat]
            # calcolo Subtotale IVA per ogni aliquota
            subtotal['vat'][vat] = subtotal['total'][vat]*vat.aliquot_perc/100
            total['vat'] += subtotal['vat'][vat]
            # creo nuovo oggetto VatRow
            vatrow = VatRow()
            vatrow.quotation = quotation
            vatrow.vat = vat
            vatrow.total_income = str(subtotal['income'][vat])
            vatrow.total_fee = str(subtotal['fee'][vat])
            vatrow.total_discount = str(subtotal['discount'][vat])
            vatrow.total_vat = str(subtotal['vat'][vat])
            vatrow.save()

        total['totals'] = total['total']+total['vat']

        quotation.total_income = str(total['income'])
        quotation.total_fee = str(total['fee'])
        quotation.total_discount = str(total['discount'])
        quotation.total_vat = str(total['vat'])
        quotation.save()

    def get_description(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.long_description or (
            "{} {}".format(self.product.code, self.product.name))

    def get_description_wo_code(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.product.name

    def get_long_description(self):
        """
        Restituisce la descrizione estesa della riga o la descrizione estesa del prodotto
        """
        return self.long_description or self.product.full_description

    def get_net_price(self):
        """
        restituisce il prezzo unitario con sconto e spesa aggiuntivi
        """
        return self.price - self.discount + self.fee
    net_price = property(get_net_price)
    get_net_price.short_description = _("Net Price")

    def get_total_income(self):
        return self.quantity * self.price
    total_income = property(get_total_income)
    get_total_income.short_description = _("Total Income")

    def get_total_fee(self):
        return self.quantity * self.fee
    total_fee = property(get_total_fee)
    get_total_fee.short_description = _("Total Fee")

    def get_total_discount(self):
        return self.quantity * self.discount
    total_discount = property(get_total_discount)
    get_total_discount.short_description = _("Total Discount")

    def get_total(self):
        return self.total_income + self.total_fee - self.total_discount
    total = property(get_total)
    get_total.short_description = _("Total")

    def get_cost(self):
        return self.labor_cost + self.material_cost + self.services_cost
    cost = property(get_cost)

    def get_profit(self):
        return self.price - self.costs
    profit = property(get_profit)

    def get_total_material_cost(self):
        """
        restituisce il costo totale dei materiali
        """
        return self.material_cost * self.quantity

    def get_total_service_cost(self):
        """
        restituisce il costo totale dei servizi
        """
        return self.services_cost * self.quantity

    def get_total_labor_cost(self):
        """
        restituisce il costo totale dei manodopera
        """
        return self.labor_cost * self.quantity

    def get_total_cost(self):
        """
        restituisce il totale dei costi
        """
        return self.costs * self.quantity

    def get_total_profit(self):
        """
        restituisce l'utile totale
        """
        return self.profit * self.quantity

    def delete(self):
        super(AbstractQuotationRow, self).delete()
        self.update_totals()

    def delete_not_update(self):
        super(AbstractQuotationRow, self).delete()

    def can_be_deleted(self):
        # if self.order_done or self.accepted or self.submitted:
        #     return False
        return True

    def can_be_modified(self):
        return True

    def can_be_cloned(self):
        return True

    def can_be_revisioned(self):
        return True


class AbstractVatRow(models.Model):
    # quotation = models.ForeignKey(
    #     Quotation,
    #     related_name="vattotalquotationrows",
    #     db_index=True,
    #     verbose_name=_("Quotation"),
    # )
    quotation = DeferredForeignKey(
        related_name="vattotalquotationrows",
        db_index=True,
        verbose_name=_("Quotation"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name="vattotalquotationrows",
        db_index=True,
        verbose_name=_("Vat"),
    )
    total_income = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Income"),
    )
    total_fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Fee"),
    )
    total_discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Discount"),
    )
    total_vat = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Vat"),
    )

    class Meta:
        abstract = True
        verbose_name = _("vat row")
        verbose_name_plural = _("vat rows")
