# -*- coding: utf-8 -*-

"""
.. _models:

Modello della Quotation, QuotationProject, Quotation Row

Deps: Organization, Contact, OrganizationAddress
    Currency, Delivery, Vat, OrganizationPriceList
    Product, PaymentForm, Office
    StoreCause, Project

.. autoclass:: Quotation
   :members:

.. autoclass:: QuotationRow
   :members:

.. autoclass:: QuotationProject
   :members:

"""
import datetime
import decimal

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models, transaction
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE
from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel

from jmb.currency.models import Currency
from jmb.carriage.models import Delivery
from jmb.i18n.models import TranslationModel, I18NModel
from jmb.office.models import Office
from jmb.organization.models import Organization, Contact, OrganizationAddress
from jmb.payment.models import PaymentForm
from jmb.price.models import OrganizationPriceList
from jmb.product.models import Product
from jmb.project.models import Project
from jmb.quotation.managers import (
    SupplierQuotationProjectManager, CustomerQuotationProjectManager,
    SupplierQuotationManager, CustomerQuotationManager,
    SupplierQuotationRowManager, CustomerQuotationRowManager,
    SupplierVatRowManager, CustomerVatRowManager
)
from jmb.store.models import StoreCause
from jmb.vat.models import Vat

try:                                                                                                                                           
    from django.db.transaction import set_autocommit                                                                                           
    set_autocommit = set_autocommit                                                                                                            
except ImportError:                                                                                                                            
    from django.db.transaction import commit_manually                                                                                          
    set_autocommit = commit_manually 

class QuotationProject(Project):
    pass


class SupplierQuotationProject(QuotationProject):
    objects = SupplierQuotationProjectManager()

    class Meta:
        proxy = True
        verbose_name = _('supplier quotation project')
        verbose_name_plural = _('supplier quotation projects')


class CustomerQuotationProject(QuotationProject):
    objects = CustomerQuotationProjectManager()

    class Meta:
        proxy = True
        verbose_name = _('customer quotation project')
        verbose_name_plural = _('customer quotation projects')

    def __unicode__(self):
        return unicode(self.name)

    @classmethod
    def create_from(cls, project):
        """
        Crea una QuotationProject a partire da una jmb.project.Project
        """
        new = cls(project_ptr=project)
        new.__dict__.update(project.__dict__)
        new.save()
        return new


class Quotation(DateModel, UserModel):
    owner = models.ForeignKey(
        Organization,
        related_name="owner_quotations",
        db_index=True,
        verbose_name=_("Bill to"),
    )
    recipient = models.ForeignKey(
        Organization,
        null=True, blank=True,
        related_name="recipient_quotations",
        db_index=True,
        verbose_name=_("Ship to"),
    )
    project = models.ForeignKey(
        QuotationProject,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Project"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Payment Form"),
    )
    office = models.ForeignKey(
        Office,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Office"),
    )
    contact = models.ForeignKey(
        Contact,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Contact"),
    )
    currency = models.ForeignKey(
        Currency,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Currency")
    )
    delivery = models.ForeignKey(
        Delivery,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Delivery")
    )
    vat = models.ForeignKey(
        Vat,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Vat")
    )
    #    agent = models.ForeignKey (
    #        Agent,
    #        null = True, blank = True,
    #        related_name = "quotations",
    #        db_index = True,
    #        verbose_name = _("Agent"),
    #    )
    #    commision = models.ForeignKey (
    #        Commision,
    #        blank = True, null = True,
    #        related_name = "quotations",
    #        db_index = True,
    #        verbose_name = _("Agent & Commision")
    #    )
    price_list = models.ForeignKey(
        OrganizationPriceList,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Prices List & Discount")
    )
    #    vector = models.ForeignKey (
    #        Vector,
    #        blank = True, null = True,
    #        related_name = "quotations",
    #        db_index = True,
    #        verbose_name = _("Vector")
    #    )
    store_cause = models.ForeignKey(
        StoreCause,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Store Cause")
    )
    shipping_address = models.ForeignKey(
        OrganizationAddress,
        blank=True, null=True,
        related_name="quotations",
        db_index=True,
        verbose_name=_("Shipping Address")
    )
    description = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_("Description")
    )
    full_description = HTMLField(
        blank=True,
        null=True,
        verbose_name=_("Full description"),
    )
    number = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Number"),
        db_column="qnumber"
    )
    revision = models.IntegerField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Revision"),
    )
    revision_year = models.IntegerField(
        verbose_name=_("First revision year")
    )
    date_q = models.DateField(
        db_index=True,
        verbose_name=_("Date")
    )
    # preventivo già trasformato in ordine
    order_done = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Ordered"),
    )
    # data prevista consegna
    date_delivery = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Date Delivery"),
    )
    # campi condizioni generali
    # validita offerta
    quote_firm_for = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Quote Firm For"),
    )
    # tempi di consegna
    delivery_window = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("Delivery Window"),
    )
    # fatto sopralluogo
    inspected = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Inspected"),
    )
    # data sopralluogo
    date_inspected = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Inspected Date"),
    )
    # preventivo presentato
    submitted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Submitted"),
    )
    # data presentazione
    date_submitted = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Submitted Date"),
    )
    # preventivo accettato
    accepted = models.BooleanField(
        default=False,
        db_index=True,
        verbose_name=_("Accepted"),
    )
    # data accettazione
    date_accepted = models.DateField(
        blank=True, null=True,
        db_index=True,
        verbose_name=_("Accepted Date"),
    )
    total_income = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Income"),
    )
    total_fee = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Fee"),
    )
    total_discount = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Discount"),
    )
    total_vat = models.DecimalField(
        default=0,
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total VAT"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )

    class Meta:
        verbose_name = _("quotation")
        verbose_name_plural = _("quotations")

    def __unicode__(self):
        return u"{date}, {number}".format({'date': self.date_q, 'number': self.number})

    def save(self, *args, **kwargs):
        if not self.number:
            # number
            from jmb.store.utils import create_update_storedocumentnumber
            self.number = create_update_storedocumentnumber(
                'q', curr_date=self.date_q, store_document=self.store_cause.store_document
            )

        if not self.recipient:
            self.recipient = self.owner

        if not self.revision:
            self.revision = 0

        if not self.revision_year:
            self.revision_year = datetime.date.today().year

        # date inspected
        if self.date_inspected and not self.inspected:
            self.inspected = True
        if not self.date_inspected:
            self.inspected = False

        # date accepted
        if self.date_accepted and not self.accepted:
            self.accepted = True
        if not self.date_accepted:
            self.accepted = False

        # date submitted
        if self.date_submitted and not self.submitted:
            self.submitted = True
        if not self.date_submitted:
            self.submitted = False

        super(Quotation, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if self.can_be_deleted:
            return super(Quotation, self).delete()

    def can_be_deleted(self):
        if self.order_done or self.accepted or self.submitted:
            return False
        return True

    @set_autocommit
    def clone(self):
        try:
            new_kwargs = dict([
                (fld.name, getattr(self, fld.name)) for fld in self._meta.fields if fld.name != 'id'
            ])
            # return self.__class__.objects.create(**new_kwargs)
            new_obj = self.__class__(**new_kwargs)
            new_obj.save()
            transaction.commit()
            return True, new_obj
        except Exception, e:
            transaction.rollback()
            return False, e

    @set_autocommit
    def do_revision(self, request):
        try:
            created, new_obj = self.clone()
            if created:
                new_obj.order_done = False
                new_obj.date_delivery = None
                new_obj.submitted = False
                new_obj.date_submitted = None
                new_obj.accepted = False
                new_obj.date_accepted = None
                new_obj.date_q = datetime.date.today()

                # controllo revisioni precedenti con:
                # - stesso numero
                # - stesso anno di prima revisione
                # - stesso intestatario
                revision = self.__class__.objects.filter(
                    number=self.number, revision_year=self.revision_year,
                    owner=self.owner
                ).count()
                new_obj.revision += 1
                new_obj.save()
                return True, new_obj
        except Exception, e:
            transaction.rollback()
            return False, e

    def get_number(self):
        """
        Return formatted number
        """
        number = ""
        if self.number:
            number = str(self.number).rjust(settings.QUOTATION_ZEROFILL_NUMBER, '0')
        if self.revision == 0:
            return "{}-{}".format(number, self.revision_year)
        return "{}/{}-{}".format(number, self.revision, self.revision_year)

    get_number.short_description = _("Number")

    def get_total(self):
        return (
            self.total_income - self.total_discount +
            self.total_fee + self.total_vat
        )

    def get_totals(self):
        totals = dict()
        totals['income'] = decimal.Decimal(self.total_income)
        totals['fee'] = decimal.Decimal(self.total_fee)
        totals['discount'] = decimal.Decimal(self.total_discount)
        totals['vat'] = decimal.Decimal(self.total_vat)
        totals['total_no_vat'] = (
            decimal.Decimal(self.total_income) +
            decimal.Decimal(self.total_fee) -
            decimal.Decimal(self.total_discount)
        )
        totals['total'] = totals['total_no_vat'] + decimal.Decimal(self.total_vat)
        return totals

    totals = property(get_totals)

    def update_totals(self):
        try:
            first_quotationrow = self.rows.all()[0]
            first_quotationrow.update_totals()
        except Exception:
            pass

    def default_invoice_address(self):
        """
        restituisce il primo indirizzo di fatturazione
        """
        invoice_addresses = self.owner.get_addresses().filter(
            address_scopes__is_invoice_address=True)
        if invoice_addresses:
            return invoice_addresses[0]
        return None

    def default_invoice_address_dict(self):
        """
        restituisce un dizionario con le prime 3 righe dell'indirizzo di fatturazione
        """
        address = self.default_invoice_address()
        address_dict = dict()
        address_dict["row1"] = "{} {}, {}".format(
            address.get_address_type_display() or "", address.name, address.number
        )
        address_dict["row2"] = "{} {} ({})".format(address.zip_code, address.locality, address.province)
        address_dict["row3"] = "{}".format(address.country)
        return address_dict

    def default_shipping_address(self):
        """
        restituisce il primo indirizzo di spedizione
        """
        shipping_addresses = self.owner.get_addresses().filter(
            address_scopes__is_shipping_address=True
        )
        if shipping_addresses:
            return shipping_addresses[0]
        return None

    def shipping_address_dict(self):
        address_dict = dict()
        address_dict["row1"] = "{} {}, {}".format(
            self.shipping_address.get_address_type_display() or "",
            self.shipping_address.name,
            self.shipping_address.number
        )
        address_dict["row2"] = "{} {} ({})".format(
            self.shipping_address.zip_code,
            self.shipping_address.locality,
            self.shipping_address.province
        )
        address_dict["row3"] = "{}".format(self.shipping_address.country)
        return address_dict

    def send_email(self):
        pass


class CustomerQuotation(Quotation):
    objects = CustomerQuotationManager()

    class Meta:
        proxy = True
        verbose_name = _("customer quotation")
        verbose_name_plural = _("customer quotations")
        permissions = (
            ('view_customer_quotation', 'Can VIEW Customer quotation'),
            ('list_customer_quotation', 'Can LIST Customer quotation'),
        )


class SupplierQuotation(Quotation):
    objects = SupplierQuotationManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier quotation")
        verbose_name_plural = _("supplier quotations")
        permissions = (
            ('view_supplier_quotation', 'Can VIEW Supplier quotation'),
            ('list_supplier_quotation', 'Can LIST Supplier quotation'),
        )


class QuotationRow(DateModel, UserModel, OrderedModel):
    quotation = models.ForeignKey(
        Quotation,
        related_name="rows",
        db_index=True,
        verbose_name=_("Quotation"),
    )
    product = models.ForeignKey(
        Product,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Product"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Vat"),
    )
    store_cause = models.ForeignKey(
        StoreCause,
        related_name="quotationrows",
        db_index=True,
        verbose_name=_("Store Cause"),
    )
    project = models.ForeignKey(
        QuotationProject,
        null=True, blank=True,
        db_index=True,
        related_name="quotationrows",
        verbose_name=_("Project"),
    )
    quantity = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Quantity"),
    )
    m_unit = models.CharField(
        max_length=2,
        choices=settings.PRODUCT_PRODUCT_UNIT_TYPE,
        default="pz",
        db_index=True,
        verbose_name=_("Unit"),
    )
    price = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Price"),
    )
    # costo manodopera
    labor_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Labor cost"),
    )
    # costo materiale
    material_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Matrial cost"),
    )
    # costo dei servizi - noleggio e trasporti
    services_cost = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Services cost"),
    )
    # ricarico da applicare sui precedenti costi per determinare il prezzo
    # di vendita
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )
    fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Fee"),
    )
    discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount"),
    )
    description = models.CharField(
        max_length=200,
        blank=True, null=True,
        verbose_name=_("Description")
    )
    long_description = models.TextField(
        blank=True, null=True,
        verbose_name=_("Long Description"),
    )
    to_print = models.BooleanField(
        db_index=True,
        default=True,
        verbose_name=_("To Print"),
    )
    # riepilogo
    # campo booleano utilizzato per inserire la riga nel report di riepilogo
    summary = models.BooleanField(
        db_index=True,
        default=True,
        verbose_name=_("Summary"),
    )

    class Meta:
        verbose_name = _("quotation row")
        verbose_name_plural = _("quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']

    def __unicode__(self):
        return u"{quotation}, {product}".format({
            'quotation': self.quotation, 'product': self.product
        })

    def save(self, force_insert=False, force_update=False):
        super(QuotationRow, self).save(force_insert=force_insert, force_update=force_update)
        self.update_totals()

    def save_not_update(self, force_insert=False, force_update=False):
        super(QuotationRow, self).save(force_insert=force_insert, force_update=force_update)

    def delete(self):
        super(QuotationRow, self).delete()
        self.update_totals()

    def update_totals(self):
        from jmb.quotation.models import VatRow
        quotation = self.quotation
        rows = self.quotation.rows.all()
        VatRow.objects.filter(quotation=quotation).delete()
        total = {'income': 0, 'fee': 0, 'discount': 0, 'total': 0, 'vat': 0}
        subtotal = {'income': {}, 'fee': {}, 'discount': {}, 'vat': {}, 'total': {}}
        for row in rows:
            subtotal['income'][row.vat] = subtotal['income'].get(row.vat, 0)
            subtotal['income'][row.vat] += row.total_income

            subtotal['fee'][row.vat] = subtotal['fee'].get(row.vat, 0)
            subtotal['fee'][row.vat] += row.total_fee
            subtotal['discount'][row.vat] = subtotal['discount'].get(row.vat, 0)
            subtotal['discount'][row.vat] += row.total_discount

            subtotal['total'][row.vat] = subtotal['total'].get(row.vat, 0)
            subtotal['total'][row.vat] += row.total

        for vat in subtotal['income'].keys():
            # calcolo Totali Procedura
            total['income'] += subtotal['income'][vat]
            total['fee'] += subtotal['fee'][vat]
            total['discount'] += subtotal['discount'][vat]
            total['total'] += subtotal['total'][vat]
            # calcolo Subtotale IVA per ogni aliquota
            subtotal['vat'][vat] = subtotal['total'][vat]*vat.aliquot_perc/100
            total['vat'] += subtotal['vat'][vat]
            # creo nuovo oggetto VatRow
            vatrow = VatRow()
            vatrow.quotation = quotation
            vatrow.vat = vat
            vatrow.total_income = str(subtotal['income'][vat])
            vatrow.total_fee = str(subtotal['fee'][vat])
            vatrow.total_discount = str(subtotal['discount'][vat])
            vatrow.total_vat = str(subtotal['vat'][vat])
            vatrow.save()

        total['totals'] = total['total']+total['vat']

        quotation.total_income = str(total['income'])
        quotation.total_fee = str(total['fee'])
        quotation.total_discount = str(total['discount'])
        quotation.total_vat = str(total['vat'])
        quotation.save()

    def get_description(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.long_description or "{} {}".format(self.product.code, self.product.name)

    def get_description_wo_code(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.product.name

    def get_long_description(self):
        """
        Restituisce la descrizione estesa della riga o la descrizione estesa del prodotto
        """
        return self.long_description or self.product.full_description

    def get_net_price(self):
        """
        restituisce il prezzo unitario con sconto e spesa aggiuntivi
        """
        return self.price - self.discount + self.fee
    net_price = property(get_net_price)
    get_net_price.short_description = _("Net Price")

    def get_total_income(self):
        return self.quantity * self.price
    total_income = property(get_total_income)
    get_total_income.short_description = _("Total Income")

    def get_total_fee(self):
        return self.quantity * self.fee
    total_fee = property(get_total_fee)
    get_total_fee.short_description = _("Total Fee")

    def get_total_discount(self):
        return self.quantity * self.discount
    total_discount = property(get_total_discount)
    get_total_discount.short_description = _("Total Discount")

    def get_total(self):
        return self.total_income + self.total_fee - self.total_discount
    total = property(get_total)
    get_total.short_description = _("Total")

    def get_cost(self):
        return self.labor_cost + self.material_cost + self.services_cost
    cost = property(get_cost)

    def get_profit(self):
        return self.price - self.costs
    profit = property(get_profit)

    def get_total_material_cost(self):
        """
        restituisce il costo totale dei materiali
        """
        return self.material_cost * self.quantity

    def get_total_service_cost(self):
        """
        restituisce il costo totale dei servizi
        """
        return self.services_cost * self.quantity

    def get_total_labor_cost(self):
        """
        restituisce il costo totale dei manodopera
        """
        return self.labor_cost * self.quantity

    def get_total_cost(self):
        """
        restituisce il totale dei costi
        """
        return self.costs * self.quantity

    def get_total_profit(self):
        """
        restituisce l'utile totale
        """
        return self.profit * self.quantity


class CustomerQuotationRow(QuotationRow):
    objects = CustomerQuotationRowManager()

    class Meta:
        proxy = True
        verbose_name = _("customer quotation row")
        verbose_name_plural = _("customer quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']
        permissions = (
            ('view_customer_quotationrow', 'Can VIEW Customer quotation row'),
            ('list_customer_quotationrow', 'Can LIST Customer quotation row'),
        )


class SupplierQuotationRow(QuotationRow):
    objects = SupplierQuotationRowManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier quotation row")
        verbose_name_plural = _("supplier quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']
        permissions = (
            ('view_supplier_quotationrow', 'Can VIEW Supplier quotation row'),
            ('list_supplier_quotationrow', 'Can LIST Supplier quotation row'),
        )


class VatRow(DateModel, UserModel, StatusModel):
    quotation = models.ForeignKey(
        Quotation,
        related_name="vattotalquotationrows",
        db_index=True,
        verbose_name=_("Quotation"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name="vattotalquotationrows",
        db_index=True,
        verbose_name=_("Vat"),
    )
    total_income = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Income"),
    )
    total_fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Fee"),
    )
    total_discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Discount"),
    )
    total_vat = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        verbose_name=_("Total Vat"),
    )

    class Meta:
        verbose_name = _("vat row")
        verbose_name_plural = _("vat rows")
        ordering = ('vat__ordering', 'vat__code')

        def __unicode__(self):
            return u"{} {} {} {}".format(
                self.quotation, self.vat, self.total_income, self.total_vat
            )


class CustomerVatRow(VatRow):
    objects = CustomerVatRowManager()

    class Meta:
        proxy = True
        verbose_name = _("customer vat row")
        verbose_name_plural = _("customer vat rows")
        ordering = ('vat__ordering', 'vat__code')
        permissions = (
            ('view_customer_vatrow', 'Can VIEW Customer quotation row'),
            ('list_customer_vatrow', 'Can LIST Customer quotation row'),
        )


class SupplierVatRow(VatRow):
    objects = SupplierVatRowManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier vat row")
        verbose_name_plural = _("supplier vat rows")
        ordering = ('vat__ordering', 'vat__code')
        permissions = (
            ('view_supplier_vatrow', 'Can VIEW Supplier quotation row'),
            ('list_supplier_vatrow', 'Can LIST Supplier quotation row'),
        )
